package x86_test

import (
	"testing"

	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/x86"
	"git.akyoto.dev/go/assert"
)

func TestAndRegisterNumber(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   int
		Code     []byte
	}{
		{x86.RAX, 1, []byte{0x48, 0x83, 0xE0, 0x01}},
		{x86.RCX, 1, []byte{0x48, 0x83, 0xE1, 0x01}},
		{x86.RDX, 1, []byte{0x48, 0x83, 0xE2, 0x01}},
		{x86.RBX, 1, []byte{0x48, 0x83, 0xE3, 0x01}},
		{x86.RSP, 1, []byte{0x48, 0x83, 0xE4, 0x01}},
		{x86.RBP, 1, []byte{0x48, 0x83, 0xE5, 0x01}},
		{x86.RSI, 1, []byte{0x48, 0x83, 0xE6, 0x01}},
		{x86.RDI, 1, []byte{0x48, 0x83, 0xE7, 0x01}},
		{x86.R8, 1, []byte{0x49, 0x83, 0xE0, 0x01}},
		{x86.R9, 1, []byte{0x49, 0x83, 0xE1, 0x01}},
		{x86.R10, 1, []byte{0x49, 0x83, 0xE2, 0x01}},
		{x86.R11, 1, []byte{0x49, 0x83, 0xE3, 0x01}},
		{x86.R12, 1, []byte{0x49, 0x83, 0xE4, 0x01}},
		{x86.R13, 1, []byte{0x49, 0x83, 0xE5, 0x01}},
		{x86.R14, 1, []byte{0x49, 0x83, 0xE6, 0x01}},
		{x86.R15, 1, []byte{0x49, 0x83, 0xE7, 0x01}},

		{x86.RAX, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE0, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RCX, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE1, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RDX, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE2, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RBX, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE3, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RSP, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE4, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RBP, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE5, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RSI, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE6, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.RDI, 0x7FFFFFFF, []byte{0x48, 0x81, 0xE7, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R8, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE0, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R9, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE1, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R10, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE2, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R11, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE3, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R12, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE4, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R13, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE5, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R14, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE6, 0xFF, 0xFF, 0xFF, 0x7F}},
		{x86.R15, 0x7FFFFFFF, []byte{0x49, 0x81, 0xE7, 0xFF, 0xFF, 0xFF, 0x7F}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("and %s, %x", pattern.Register, pattern.Number)
		code := x86.AndRegisterNumber(nil, pattern.Register, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestAndRegisterRegister(t *testing.T) {
	usagePatterns := []struct {
		Left  cpu.Register
		Right cpu.Register
		Code  []byte
	}{
		{x86.RAX, x86.R15, []byte{0x4C, 0x21, 0xF8}},
		{x86.RCX, x86.R14, []byte{0x4C, 0x21, 0xF1}},
		{x86.RDX, x86.R13, []byte{0x4C, 0x21, 0xEA}},
		{x86.RBX, x86.R12, []byte{0x4C, 0x21, 0xE3}},
		{x86.RSP, x86.R11, []byte{0x4C, 0x21, 0xDC}},
		{x86.RBP, x86.R10, []byte{0x4C, 0x21, 0xD5}},
		{x86.RSI, x86.R9, []byte{0x4C, 0x21, 0xCE}},
		{x86.RDI, x86.R8, []byte{0x4C, 0x21, 0xC7}},
		{x86.R8, x86.RDI, []byte{0x49, 0x21, 0xF8}},
		{x86.R9, x86.RSI, []byte{0x49, 0x21, 0xF1}},
		{x86.R10, x86.RBP, []byte{0x49, 0x21, 0xEA}},
		{x86.R11, x86.RSP, []byte{0x49, 0x21, 0xE3}},
		{x86.R12, x86.RBX, []byte{0x49, 0x21, 0xDC}},
		{x86.R13, x86.RDX, []byte{0x49, 0x21, 0xD5}},
		{x86.R14, x86.RCX, []byte{0x49, 0x21, 0xCE}},
		{x86.R15, x86.RAX, []byte{0x49, 0x21, 0xC7}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("and %s, %s", pattern.Left, pattern.Right)
		code := x86.AndRegisterRegister(nil, pattern.Left, pattern.Right)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
